@@ -57,6 +57,7 @@ extern "C" const char *Version_string;
                                                      /* either encoded by their glyph names or if */
                                                      /* there is no name then we use &#nnn;       */
 typedef enum {CENTERED, LEFT, RIGHT, INLINE} TAG_ALIGNMENT;
+typedef enum {col_tag, tab_tag, tab0_tag, none} colType;
 
 #undef DEBUG_TABLES
 
@@ -254,16 +255,25 @@ int style::operator!=(const style &s) const
 
 struct char_block {
   enum { SIZE = 256 };
-  char          buffer[SIZE];
+  char         *buffer;
   int           used;
   char_block   *next;
 
   char_block();
+  char_block::char_block(int length);
 };
 
 char_block::char_block()
+: buffer(NULL), used(0), next(0)
+{
+}
+
+char_block::char_block(int length)
 : used(0), next(0)
 {
+  buffer = (char *)malloc(max(length, char_block::SIZE));
+  if (buffer == NULL)
+    fatal("out of memory error");
 }
 
 class char_buffer {
@@ -300,18 +310,14 @@ char *char_buffer::add_string (const char *s, unsigned int length)
     return NULL;
 
   if (tail == 0) {
-    tail = new char_block;
+    tail = new char_block(length+1);
     head = tail;
   } else {
     if (tail->used + length+1 > char_block::SIZE) {
-      tail->next = new char_block;
-      tail       = tail->next;
+      tail->next  = new char_block(length+1);
+      tail        = tail->next;
     }
   }
-  // at this point we have a tail which is ready for the string.
-  if (tail->used + length+1 > char_block::SIZE) {
-    fatal("need to increase char_block::SIZE");
-  }
 
   old_used = tail->used;
   do {
@@ -366,7 +372,6 @@ public:
   int  is_eol              (void);
   int  is_auto_img         (void);
   int  is_br               (void);
-  int  is_br_ni            (void);
   int  is_in               (void);
   int  is_po               (void);
   int  is_ti               (void);
@@ -687,19 +692,6 @@ int text_glob::is_auto_img (void)
  */
 
 int text_glob::is_br (void)
-{
-  return( is_a_tag() && ((strcmp ("html-tag:.br", text_string) == 0) ||
-			 (strncmp("html-tag:.sp", text_string, 11) == 0) ||
-			 (strcmp ("html-tag:.ce", text_string) == 0) ||
-			 (strncmp ("html-tag:.in", text_string, 11) == 0)) );
-}
-
-/*
- *  is_br_ni - returns TRUE if the glob is a tag containing a .br
- *             or an implied .br, but not an .in
- */
-
-int text_glob::is_br_ni (void)
 {
   return( is_a_tag() && ((strcmp ("html-tag:.br", text_string) == 0) ||
 			 (strncmp("html-tag:.sp", text_string, 11) == 0) ||
@@ -1071,6 +1063,7 @@ void list::insert (text_glob *in)
     element_list *t = new element_list(in, ptr->lineno, ptr->minv, ptr->minh, ptr->maxv, ptr->maxh);
     if (ptr == tail)
       tail = t;
+    ptr->right->left = t;
     t->right = ptr->right;
     ptr->right = t;
     t->left = ptr;
@@ -1299,6 +1292,7 @@ void page::dump_page(void)
   glyphs.move_to(old_pos);
   printf("\ndebugging end\n\n");
   printf("\n-->\n");
+  fflush(stdout);
 #endif
 }
 
@@ -1529,7 +1523,7 @@ class html_printer : public printer {
   font *make_bold                     (font *f);
   int   overstrike                    (int index, const char *name, const environment *env, int w);
   void  do_body                       (void);
-  int   next_horiz_pos                (int nf);
+  int   next_horiz_pos                (text_glob *g, int nf);
   void  lookahead_for_tables          (void);
   void  insert_tab_te                 (void);
   text_glob *insert_tab_ts            (text_glob *where);
@@ -1546,6 +1540,8 @@ class html_printer : public printer {
   void calc_po_in                     (text_glob *g, int nf);
   void remove_tabs                    (void);
   void remove_courier_tabs            (void);
+  void update_min_max                 (colType type_of_col, int *minimum, int *maximum, text_glob *g);
+  void add_table_end                  (char *debug_string);
   // ADD HERE
 
 public:
@@ -2219,16 +2215,14 @@ void html_printer::do_tab_ts (text_glob *g)
   html_table *t = g->get_table();
 
   if (t != NULL) {
-    int need_space = ((! current_paragraph->ever_emitted_text()) &&
-		      current_paragraph->starts_with_space());
-
     current_paragraph->done_pre();
     current_paragraph->done_para();
 
     html.simple_comment("TABS");
+
     t->set_linelength(max_linelength);
     t->add_indent(pageoffset);
-    t->emit_table_header(need_space);
+    t->emit_table_header(FALSE);
   }
 
   table = t;
@@ -2368,6 +2362,9 @@ void html_printer::troff_tag (text_glob *g)
   } else if (strncmp(t, ".vs", 3) == 0) {
     char *a = (char *)t+3;
     do_verticalspacing(a);
+  } else if (strncmp(t, ".ps", 3) == 0) {
+    char *a = (char *)t+3;
+    do_pointsize(a);
   } else if (strcmp(t, ".links") == 0) {
     do_links();
   } else if (strcmp(t, ".no-auto-rule") == 0) {
@@ -2436,10 +2433,12 @@ void html_printer::flush_globs (void)
 
 int html_printer::calc_nf (text_glob *g, int nf)
 {
-  if (g->is_fi())
-    return FALSE;
-  if (g->is_nf())
-    return TRUE;
+  if (g != NULL) {
+    if (g->is_fi())
+      return FALSE;
+    if (g->is_nf())
+      return TRUE;
+  }
   return nf;
 }
 
@@ -2468,16 +2467,19 @@ void html_printer::calc_po_in (text_glob *g, int nf)
  *                   -1 is returned if it doesn't exist.
  */
 
-int html_printer::next_horiz_pos (int nf)
+int html_printer::next_horiz_pos (text_glob *g, int nf)
 {
   int next     = -1;
-  text_glob *g = page_contents->glyphs.get_data();
 
-  if (g->is_br() || (nf && g->is_eol()))
+  if ((g != NULL) && (g->is_br() || (nf && g->is_eol())))
     if (! page_contents->glyphs.is_empty()) {
-      page_contents->glyphs.move_right();
-      next = g->minh;
-      page_contents->glyphs.move_left();
+      page_contents->glyphs.move_right_get_data();
+      if (g == NULL)
+	page_contents->glyphs.start_from_head();
+      else {
+	next = g->minh;
+	page_contents->glyphs.move_left();
+      }
     }
   return next;
 }
@@ -2641,6 +2643,43 @@ void html_printer::insert_tab0_foreach_tab (void)
   }
 }
 
+/*
+ *  update_min_max - updates the extent of a column, given the left and right
+ *                   extents of a glyph, g.
+ */
+
+void html_printer::update_min_max (colType type_of_col, int *minimum, int *maximum, text_glob *g)
+{
+  switch (type_of_col) {
+    
+  case tab_tag:
+    break;
+  case tab0_tag:
+    *minimum = g->minh;
+    break;
+  case col_tag:
+    *minimum = g->minh;
+    *maximum = g->maxh;
+    break;
+  default:
+    break;
+  }
+}
+
+/*
+ *  add_table_end - moves left one glyph, adds a table end tag and adds a
+ *                  debugging string.
+ */
+
+void html_printer::add_table_end (char *debug_string)
+{
+  page_contents->glyphs.move_left();
+  insert_tab_te();
+#if defined(DEBUG_TABLES)
+  page_contents->insert_tag(string(debug_string));
+#endif
+}
+
 /*
  *  lookahead_for_tables - checks for .col tags and inserts table start/end tags
  */
@@ -2651,7 +2690,7 @@ void html_printer::lookahead_for_tables (void)
   text_glob  *start_of_line  = NULL;
   text_glob  *start_of_table = NULL;
   text_glob  *last           = NULL;
-  enum {col_tag, tab_tag, tab0_tag, none} type_of_col = none;
+  colType     type_of_col    = none;
   int         left           = 0;
   int         found_col      = FALSE;
   int         seen_text      = FALSE;
@@ -2662,6 +2701,7 @@ void html_printer::lookahead_for_tables (void)
   const char *tab_defs       = NULL;
   char        align          = 'L';
   int         nf             = FALSE;
+  int         old_pageoffset = pageoffset;
 
   remove_courier_tabs();
   page_contents->dump_page();
@@ -2671,7 +2711,11 @@ void html_printer::lookahead_for_tables (void)
     page_contents->glyphs.start_from_head();
     g = page_contents->glyphs.get_data();
     do {
-#if 0
+#if defined(DEBUG_TABLES)
+      fprintf(stderr, " [") ;
+      fprintf(stderr, g->text_string) ;
+      fprintf(stderr, "] ") ;
+      fflush(stderr);
       if (strcmp(g->text_string, "XXXXXXX") == 0)
 	stop();
 #endif
@@ -2722,22 +2766,8 @@ void html_printer::lookahead_for_tables (void)
 	ncol = 1;
 	colmin = 0;
 	colmax = table->get_tab_pos(2) + pageoffset + indentation;
-      } else if (! g->is_a_tag()) {
-	switch (type_of_col) {
-
-	case tab_tag:
-	  break;
-	case tab0_tag:
-	  colmin = g->minh;
-	  break;
-	case col_tag:
-	  colmin = g->minh;
-	  colmax = g->maxh;
-	  break;
-	default:
-	  break;
-	}
-      }
+      } else if (! g->is_a_tag())
+	update_min_max(type_of_col, &colmin, &colmax, g);
 
       if ((! g->is_a_tag()) || g->is_tab())
 	seen_text = TRUE;
@@ -2748,21 +2778,17 @@ void html_printer::lookahead_for_tables (void)
 	start_of_line = NULL;
 	seen_text = FALSE;
       } else if (g->is_ce() && (start_of_table != NULL)) {
-	page_contents->glyphs.move_left();
-	insert_tab_te();
+	add_table_end("*** CE ***");
 	start_of_table->remember_table(table);
-	page_contents->insert_tag(string("*** CE ***"));
 	start_of_table = NULL;
 	last = NULL;
       } else if (g->is_ta()) {
 	tab_defs = g->text_string;
 	if (!table->tab_stops->compatible(tab_defs)) {
 	  if (start_of_table != NULL) {
-	    page_contents->glyphs.move_left();
-	    insert_tab_te();
+	    add_table_end("*** TABS ***");
 	    start_of_table->remember_table(table);
 	    table = new html_table(&html, -1);
-	    page_contents->insert_tag(string("*** TABS ***"));
 	    start_of_table = NULL;
 	    type_of_col = none;
 	    last = NULL;
@@ -2774,14 +2800,11 @@ void html_printer::lookahead_for_tables (void)
       if (((! g->is_a_tag()) || g->is_tab()) && (start_of_table != NULL)) {
 	// we are in a table and have a glyph
 	if ((ncol == 0) || (! table->add_column(ncol, colmin, colmax, align))) {
-	  page_contents->glyphs.move_left();
-	  insert_tab_te();
-#if defined(DEBUG_TABLES)
 	  if (ncol == 0)
-	    page_contents->insert_tag(string("*** NCOL == 0 ***"));
+	    add_table_end("*** NCOL == 0 ***");
 	  else
-	    page_contents->insert_tag(string("*** CROSSED COLS ***"));
-#endif
+	    add_table_end("*** CROSSED COLS ***");
+
 	  start_of_table->remember_table(table);
 	  table = new html_table(&html, -1);
 	  start_of_table = NULL;
@@ -2793,29 +2816,34 @@ void html_printer::lookahead_for_tables (void)
       /*
        *  move onto next glob, check whether we are starting a new line
        */
-      page_contents->glyphs.move_right();
-      g = page_contents->glyphs.get_data();
+      g = page_contents->glyphs.move_right_get_data();
 
-#if defined(DEBUG_TABLES)
-      if (g->is_in() && (!seen_text))      
-	html.simple_comment("IGNORE .in");
-      else
-#endif
-	if (g->is_br_ni() || (nf && g->is_eol())) {
+      if (g == NULL) {
+	if (found_col) {
+	  page_contents->glyphs.start_from_head();
+	  last = g;
+	  found_col = FALSE;
+	}
+      } else if (g->is_br() || (nf && g->is_eol())) {
 	do {
-	  page_contents->glyphs.move_right();
-	  g = page_contents->glyphs.get_data();
+	  g = page_contents->glyphs.move_right_get_data();
 	  nf = calc_nf(g, nf);
-	} while (g->is_br_ni() || (nf && g->is_eol()));
+	} while ((g != NULL) && (g->is_br() || (nf && g->is_eol())));
 	start_of_line = g;
 	seen_text = FALSE;
 	ncol = 0;
-	left = next_horiz_pos(nf);
+	left = next_horiz_pos(g, nf);
 	if (found_col)
 	  last = g;
 	found_col = FALSE;
       }
-    } while (! page_contents->glyphs.is_equal_to_head());
+    } while ((g != NULL) && (! page_contents->glyphs.is_equal_to_head()));
+
+#if defined(DEBUG_TABLES)
+    fprintf(stderr, "finished scanning for tables\n");
+#endif
+
+    page_contents->glyphs.start_from_head();
     if (start_of_table != NULL) {
       if (last != NULL)
 	while (last != page_contents->glyphs.get_data())
@@ -2831,8 +2859,7 @@ void html_printer::lookahead_for_tables (void)
     delete table;
 
   // and reset the registers
-  pageoffset = 0;
-  pageoffset = 0;
+  pageoffset = old_pageoffset;
   indentation = 0;
   prev_indent = 0;
   end_tempindent = 0;
@@ -2851,6 +2878,11 @@ void html_printer::flush_page (void)
   
   // move onto a new page
   delete page_contents;
+#if defined(DEBUG_TABLES)
+  fprintf(stderr, "\n\n*** flushed page ***\n\n");
+
+  html.simple_comment("new page called");
+#endif
   page_contents = new page;
 }
 
@@ -2905,6 +2937,16 @@ void html_printer::end_font (const char *fontname)
     current_paragraph->done_italic();
   } else if (strcmp(fontname, "CR") == 0) {
     current_paragraph->done_tt();
+  } else if (strcmp(fontname, "CI") == 0) {
+    current_paragraph->done_italic();
+    current_paragraph->done_tt();
+  } else if (strcmp(fontname, "CB") == 0) {
+    current_paragraph->done_bold();
+    current_paragraph->done_tt();
+  } else if (strcmp(fontname, "CBI") == 0) {
+    current_paragraph->done_bold();
+    current_paragraph->done_italic();
+    current_paragraph->done_tt();
   }
 }
 
@@ -2930,6 +2972,25 @@ void html_printer::start_font (const char *fontname)
       current_paragraph->do_pre();
     }
     current_paragraph->do_tt();
+  } else if (strcmp(fontname, "CI") == 0) {
+    if ((! fill_on) && (is_courier_until_eol())) {
+      current_paragraph->do_pre();
+    }
+    current_paragraph->do_tt();
+    current_paragraph->do_italic();
+  } else if (strcmp(fontname, "CB") == 0) {
+    if ((! fill_on) && (is_courier_until_eol())) {
+      current_paragraph->do_pre();
+    }
+    current_paragraph->do_tt();
+    current_paragraph->do_bold();
+  } else if (strcmp(fontname, "CBI") == 0) {
+    if ((! fill_on) && (is_courier_until_eol())) {
+      current_paragraph->do_pre();
+    }
+    current_paragraph->do_tt();
+    current_paragraph->do_italic();
+    current_paragraph->do_bold();
   }
 }
 
@@ -3212,6 +3273,7 @@ html_printer::html_printer()
   pageoffset(0),
   indentation(0),
   prev_indent(0),
+  pointsize(0),
   line_number(0),
   background(default_background)
 {
@@ -3479,17 +3541,31 @@ void html_printer::do_body (void)
 
 html_printer::~html_printer()
 {
+#ifdef LONG_FOR_TIME_T
+  long t;
+#else
+  time_t t;
+#endif
+
   current_paragraph->flush_text();
   html.end_line();
   html.set_file(stdout);
+  html.begin_comment("Creator     : ")
+    .put_string("groff ")
+    .put_string("version ")
+    .put_string(Version_string)
+    .end_comment();
+
+  t = time(0);
+  html.begin_comment("CreationDate: ")
+    .put_string(ctime(&t), strlen(ctime(&t))-1)
+    .end_comment();
+
   /*
    *  'HTML: The definitive guide', O'Reilly, p47. advises against specifying
-   *         the dtd, so for the moment I'll leave this commented out.
-   *         If requested we could always emit it if a command line switch
-   *         was present.
-   *
-   *  fputs("<!doctype html public \"-//IETF//DTD HTML 4.0//EN\">\n", stdout);
+   *         the dtd.
    */
+  // fputs("<!doctype html public \"-//IETF//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n", stdout);
   fputs("<html>\n", stdout);
   fputs("<head>\n", stdout);
   fputs("<meta name=\"generator\" content=\"groff -Thtml, see www.gnu.org\">\n", stdout);
@@ -3501,24 +3577,6 @@ html_printer::~html_printer()
   write_title(FALSE);
   header.write_headings(stdout, FALSE);
   write_rule();
-  {
-    html.begin_comment("Creator     : ")
-       .put_string("groff ")
-       .put_string("version ")
-       .put_string(Version_string)
-       .end_comment();
-  }
-  {
-#ifdef LONG_FOR_TIME_T
-    long
-#else
-    time_t
-#endif
-    t = time(0);
-    html.begin_comment("CreationDate: ")
-      .put_string(ctime(&t), strlen(ctime(&t))-1)
-      .end_comment();
-  }
 #if defined(DEBUGGING)
   html.begin_comment("Total number of pages: ").put_string(i_to_a(no_of_printed_pages)).end_comment();
 #endif
